@@ -156,6 +156,7 @@ where
 // Scope stack variables
 
 /// Represents an unknown list of exported scopes.
+#[repr(transparent)]
 #[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct ScopeStackVariable(NonZeroU32);
 
@@ -296,6 +297,7 @@ impl ScopeStackBindings {
 // Partial symbol stacks
 
 /// A symbol with an unknown, but possibly empty, list of exported scopes attached to it.
+#[repr(C)]
 #[derive(Clone, Copy)]
 pub struct PartialScopedSymbol {
     pub symbol: Handle<Symbol>,
@@ -423,6 +425,7 @@ impl DisplayWithPartialPaths for PartialScopedSymbol {
 /// one symbol stack variable, which will appear at the end of its precondition and postcondition.
 /// So for simplicity we just leave it out.  At some point in the future we might add it in so that
 /// the symbol and scope stack formalisms and implementations are more obviously symmetric.)
+#[repr(C)]
 #[derive(Clone, Copy)]
 pub struct PartialSymbolStack {
     deque: Deque<PartialScopedSymbol>,
@@ -599,6 +602,13 @@ impl PartialSymbolStack {
             .iter_unordered(&partials.partial_symbol_stacks)
             .copied()
     }
+
+    fn ensure_both_directions(&mut self, partials: &mut PartialPaths) {
+        self.deque
+            .ensure_backwards(&mut partials.partial_symbol_stacks);
+        self.deque
+            .ensure_forwards(&mut partials.partial_symbol_stacks);
+    }
 }
 
 impl DisplayWithPartialPaths for PartialSymbolStack {
@@ -635,6 +645,7 @@ impl DisplayWithPartialPaths for PartialSymbolStack {
 
 /// A pattern that might match against a scope stack.  Consists of a (possibly empty) list of
 /// exported scopes, along with an optional scope stack variable.
+#[repr(C)]
 #[derive(Clone, Copy)]
 pub struct PartialScopeStack {
     scopes: Deque<Handle<Node>>,
@@ -812,6 +823,13 @@ impl PartialScopeStack {
     ) -> impl Display + 'a {
         display_with(self, graph, partials)
     }
+
+    fn ensure_both_directions(&mut self, partials: &mut PartialPaths) {
+        self.scopes
+            .ensure_backwards(&mut partials.partial_scope_stacks);
+        self.scopes
+            .ensure_forwards(&mut partials.partial_scope_stacks);
+    }
 }
 
 impl DisplayWithPartialPaths for PartialScopeStack {
@@ -928,6 +946,7 @@ impl PartialScopeStackBindings {
 //-------------------------------------------------------------------------------------------------
 // Edge lists
 
+#[repr(C)]
 #[derive(Clone, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct PartialPathEdge {
     pub source_node_id: NodeID,
@@ -979,6 +998,7 @@ impl DisplayWithPartialPaths for PartialPathEdge {
 
 /// The edges in a path keep track of precedence information so that we can correctly handle
 /// shadowed definitions.
+#[repr(C)]
 #[derive(Clone, Copy)]
 pub struct PartialPathEdgeList {
     edges: Deque<PartialPathEdge>,
@@ -1115,6 +1135,12 @@ impl PartialPathEdgeList {
             .iter_unordered(&partials.partial_path_edges)
             .copied()
     }
+
+    fn ensure_both_directions(&mut self, partials: &mut PartialPaths) {
+        self.edges
+            .ensure_backwards(&mut partials.partial_path_edges);
+        self.edges.ensure_forwards(&mut partials.partial_path_edges);
+    }
 }
 
 impl DisplayWithPartialPaths for PartialPathEdgeList {
@@ -1158,6 +1184,7 @@ impl DisplayWithPartialPaths for PartialPathEdgeList {
 /// (or parts of a scope symbol's attached scope list) whose contents we don't care about.  The
 /// postconditions can _also_ refer to those variables, and describe how those variable parts of
 /// the input scope stacks are carried through unmodified into the resulting scope stack.
+#[repr(C)]
 #[derive(Clone)]
 pub struct PartialPath {
     pub start_node: Handle<Node>,
@@ -1175,7 +1202,7 @@ impl PartialPath {
         graph: &StackGraph,
         partials: &mut PartialPaths,
         node: Handle<Node>,
-    ) -> PartialPath {
+    ) -> Result<PartialPath, PathResolutionError> {
         let initial_scope_stack = ScopeStackVariable::initial();
         let symbol_stack_precondition = PartialSymbolStack::empty();
         let mut symbol_stack_postcondition = PartialSymbolStack::empty();
@@ -1186,7 +1213,10 @@ impl PartialPath {
         if let Node::PushScopedSymbol(inner_node) = inner_node {
             scope_stack_precondition = PartialScopeStack::empty();
             scope_stack_postcondition = PartialScopeStack::empty();
-            scope_stack_postcondition.push_front(partials, inner_node.scope);
+            let scope = graph
+                .node_for_id(inner_node.scope)
+                .ok_or(PathResolutionError::UnknownAttachedScope)?;
+            scope_stack_postcondition.push_front(partials, scope);
             let initial_symbol = PartialScopedSymbol {
                 symbol: inner_node.symbol,
                 scopes: Some(scope_stack_postcondition),
@@ -1202,7 +1232,7 @@ impl PartialPath {
             symbol_stack_postcondition.push_front(partials, initial_symbol);
         }
 
-        PartialPath {
+        Ok(PartialPath {
             start_node: node,
             end_node: node,
             symbol_stack_precondition,
@@ -1210,7 +1240,7 @@ impl PartialPath {
             scope_stack_precondition,
             scope_stack_postcondition,
             edges: PartialPathEdgeList::empty(),
-        }
+        })
     }
 
     /// Returns whether one path shadows another.  Note that shadowing is not commutative — if path
@@ -1324,6 +1354,20 @@ impl PartialPath {
         false
     }
 
+    /// Ensures that the content of this partial path is available in both forwards and backwards
+    /// directions.
+    pub fn ensure_both_directions(&mut self, partials: &mut PartialPaths) {
+        self.symbol_stack_precondition
+            .ensure_both_directions(partials);
+        self.symbol_stack_postcondition
+            .ensure_both_directions(partials);
+        self.scope_stack_precondition
+            .ensure_both_directions(partials);
+        self.scope_stack_postcondition
+            .ensure_both_directions(partials);
+        self.edges.ensure_both_directions(partials);
+    }
+
     /// Returns a fresh scope stack variable that is not already used anywhere in this partial
     /// path.
     pub fn fresh_scope_stack_variable(&self, partials: &mut PartialPaths) -> ScopeStackVariable {
@@ -1421,7 +1465,9 @@ impl PartialPath {
             // Pushing the scoped symbol onto our postcondition indicates that using this partial
             // path would push the scoped symbol onto the path's symbol stack.
             let sink_symbol = sink.symbol;
-            let sink_scope = sink.scope;
+            let sink_scope = graph
+                .node_for_id(sink.scope)
+                .ok_or(PathResolutionError::UnknownAttachedScope)?;
             let mut attached_scopes = self.scope_stack_postcondition;
             attached_scopes.push_front(partials, sink_scope);
             let postcondition_symbol = PartialScopedSymbol {
@@ -1576,7 +1622,7 @@ impl PartialPaths {
     {
         let mut cycle_detector = CycleDetector::new();
         let mut queue = VecDeque::new();
-        queue.push_back(PartialPath::from_node(graph, self, graph.root_node()));
+        queue.push_back(PartialPath::from_node(graph, self, graph.root_node()).unwrap());
         queue.extend(
             graph
                 .nodes_for_file(file)
@@ -1586,7 +1632,7 @@ impl PartialPaths {
                     Node::ExportedScope(_) => true,
                     _ => false,
                 })
-                .map(|node| PartialPath::from_node(graph, self, node)),
+                .map(|node| PartialPath::from_node(graph, self, node).unwrap()),
         );
         while let Some(path) = queue.pop_front() {
             if !cycle_detector.should_process_path(&path, |probe| probe.cmp(graph, self, &path)) {
@@ -1675,9 +1721,9 @@ impl Path {
 /// Manages the state of a collection of partial paths built up as part of the partial-path-finding
 /// algorithm or path-stitching algorithm.
 pub struct PartialPaths {
-    partial_symbol_stacks: DequeArena<PartialScopedSymbol>,
-    partial_scope_stacks: DequeArena<Handle<Node>>,
-    partial_path_edges: DequeArena<PartialPathEdge>,
+    pub(crate) partial_symbol_stacks: DequeArena<PartialScopedSymbol>,
+    pub(crate) partial_scope_stacks: DequeArena<Handle<Node>>,
+    pub(crate) partial_path_edges: DequeArena<PartialPathEdge>,
 }
 
 impl PartialPaths {
